@@ -1,4 +1,4 @@
-/* $Id: display.cpp 14982 2007-01-01 21:57:49Z dave $ */
+/* $Id: display.cpp 16023 2007-02-23 17:17:44Z mordante $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -159,11 +159,6 @@ display::~display()
 	prune_chat_messages(true);
 }
 
-Uint32 display::rgb(Uint8 red, Uint8 green, Uint8 blue)
-{
-	return 0xFF000000 | (red << 16) | (green << 8) | blue;
-}
-
 void display::new_turn()
 {
 	const time_of_day& tod = status_.get_time_of_day();
@@ -211,6 +206,9 @@ void display::new_turn()
 
 	image::set_colour_adjustment(tod.red,tod.green,tod.blue);
 	image::set_image_mask(tod.image_mask);
+
+	invalidate_all();
+	draw();
 }
 
 void display::adjust_colours(int r, int g, int b)
@@ -219,26 +217,6 @@ void display::adjust_colours(int r, int g, int b)
 	image::set_colour_adjustment(tod.red+r,tod.green+g,tod.blue+b);
 }
 
-
-int display::x() const { return screen_.getx(); }
-int display::mapx() const { return x() - 140; }
-int display::y() const { return screen_.gety(); }
-
-const SDL_Rect& display::map_area() const
-{
-	return theme_.main_map_location(screen_area());
-}
-
-const SDL_Rect& display::minimap_area() const
-{
-	return theme_.mini_map_location(screen_area());
-}
-
-const SDL_Rect& display::unit_image_area() const
-{
-	return theme_.unit_image_location(screen_area());
-}
-
 SDL_Rect display::screen_area() const
 {
 	const SDL_Rect res = {0,0,x(),y()};
@@ -401,26 +379,15 @@ gamemap::location display::pixel_position_to_hex(int x, int y, gamemap::location
 	return res;
 }
 
-int display::get_location_x(const gamemap::location& loc) const
+void display::get_rect_hex_bounds(SDL_Rect rect, gamemap::location &topleft, gamemap::location &bottomright) const
 {
-	return map_area().x + loc.x*hex_width() - xpos_;
-}
-
-int display::get_location_y(const gamemap::location& loc) const
-{
-	return map_area().y + loc.y*zoom_ - ypos_ + (is_odd(loc.x) ? zoom_/2 : 0);
-}
-
-void display::get_visible_hex_bounds(gamemap::location &topleft, gamemap::location &bottomright) const
-{
-	const SDL_Rect& rect = map_area();
 	const int tile_width = hex_width();
 
-	topleft.x  = xpos_ / tile_width;
-	topleft.y  = (ypos_ - (is_odd(topleft.x) ? zoom_/2 : 0)) / zoom_;
+	topleft.x  = (xpos_ + rect.x) / tile_width;
+	topleft.y  = (ypos_ + rect.y - (is_odd(topleft.x) ? zoom_/2 : 0)) / zoom_;
 
-	bottomright.x  = (xpos_ + rect.w) / tile_width;
-	bottomright.y  = ((ypos_ + rect.h) - (is_odd(bottomright.x) ? zoom_/2 : 0)) / zoom_;
+	bottomright.x  = (xpos_ + rect.x + rect.w) / tile_width;
+	bottomright.y  = ((ypos_ + rect.y + rect.h) - (is_odd(bottomright.x) ? zoom_/2 : 0)) / zoom_;
 
 	if(topleft.x > -1) {
 		topleft.x--;
@@ -466,23 +433,63 @@ void display::scroll(int xmove, int ymove)
 	xpos_ += xmove;
 	ypos_ += ymove;
 	bounds_check_position();
+	const int dx = orig_x - xpos_; // dx = -xmove
+	const int dy = orig_y - ypos_; // dy = -ymove
 
 	//only invalidate if we've actually moved
-	if(orig_x != xpos_ || orig_y != ypos_) {
-		map_labels_.scroll(orig_x - xpos_, orig_y - ypos_);
-		font::scroll_floating_labels(orig_x - xpos_, orig_y - ypos_);
-		invalidate_all();
+	if(dx == 0 && dy == 0)
+		return;
+
+	map_labels_.scroll(dx, dy);
+	font::scroll_floating_labels(dx, dy);
+	
+	surface screen(screen_.getSurface());
+	
+	SDL_Rect dstrect = map_area();
+	dstrect.x += dx;
+	dstrect.y += dy;
+	dstrect = intersect_rects(dstrect, map_area());
+
+	SDL_Rect srcrect = dstrect;
+	srcrect.x -= dx;
+	srcrect.y -= dy;
+	
+	SDL_BlitSurface(screen,&srcrect,screen,&dstrect);
+	
+	//invalidate locations in the newly visible rects
+
+	if (dy != 0) {
+		SDL_Rect r = map_area();
+		if (dy < 0) {
+			r.y += r.h-abs(dy);
+		}
+		r.h = abs(dy);
+		invalidate_locations_in_rect(r);
 	}
+	if (dx != 0) {
+		SDL_Rect r = map_area();
+		if (dx < 0) {
+			r.x += r.w-abs(dx);
+		}
+		r.w = abs(dx);
+		invalidate_locations_in_rect(r);
+	}
+
+	update_rect(map_area());
+
+	redrawMinimap_ = true;
 }
 
-int display::hex_size() const
+void display::invalidate_locations_in_rect(SDL_Rect r)
 {
-	return zoom_;
-}
-
-int display::hex_width() const
-{
-	return (zoom_*3)/4;
+	gamemap::location topleft, bottomright;
+	get_rect_hex_bounds(r, topleft, bottomright);
+	for (int x = topleft.x; x <= bottomright.x; ++x) {
+		for (int y = topleft.y; y <= bottomright.y; ++y) {
+			gamemap::location loc(x, y);
+			invalidate(loc);
+		}
+	}
 }
 
 double display::zoom(int amount)
@@ -514,11 +521,6 @@ double display::zoom(int amount)
 	return double(zoom_)/double(DefaultZoom);
 }
 
-void display::default_zoom()
-{
-	zoom(DefaultZoom - zoom_);
-}
-
 void display::screenshot()
 {
 	std::string datadir = get_screenshot_dir();
@@ -542,6 +544,11 @@ void display::screenshot()
 	SDL_SaveBMP(screen_.getSurface().get(), name.c_str());
 }
 
+void display::default_zoom()
+{ 
+	zoom(DefaultZoom - zoom_); 
+}
+
 void display::scroll_to_tile(int x, int y, SCROLL_TYPE scroll_type, bool check_fogged)
 {
 	if(screen_.update_locked() || (check_fogged && fogged(x,y))) {
@@ -557,8 +564,6 @@ void display::scroll_to_tile(int x, int y, SCROLL_TYPE scroll_type, bool check_f
 	const int xpos = get_location_x(loc);
 	const int ypos = get_location_y(loc);
 	if ((scroll_type == ONSCREEN) && !outside_area(map_area(),xpos,ypos)) {
-		invalidate_all();
-		draw();
 		return;
 	}
 
@@ -573,7 +578,7 @@ void display::scroll_to_tile(int x, int y, SCROLL_TYPE scroll_type, bool check_f
 
 	int num_moves = (abs(xmove) > abs(ymove) ? abs(xmove):abs(ymove))/speed;
 
-	if(scroll_type == WARP || turbo()) {
+	if(scroll_type == WARP || turbo() || num_moves == 0) {
 		num_moves = 1;
 	}
 
@@ -587,12 +592,9 @@ void display::scroll_to_tile(int x, int y, SCROLL_TYPE scroll_type, bool check_f
 			continue;
 		}
 
-	invalidate_all();
 		draw();
 	}
 
-	invalidate_all();
-	draw();
 }
 
 void display::scroll_to_tiles(int x1, int y1, int x2, int y2,
@@ -819,6 +821,8 @@ void display::draw(bool update,bool force)
 	//log_scope("Drawing");
 	invalidate_animations();
 
+	process_reachmap_changes();
+
 	if(!panelsDrawn_) {
 		surface const screen(screen_.getSurface());
 
@@ -1857,14 +1861,42 @@ void display::highlight_another_reach(const paths &paths_list)
 	// Fold endpoints of routes into reachability map.
 	for (r = paths_list.routes.begin(); r != paths_list.routes.end(); ++r) {
 		reach_map_[r->first]++;
-		invalidate(r->first);
 	}
+	reach_map_changed_ = true;
 }
 
 void display::unhighlight_reach()
 {
 	reach_map_ = reach_map();
-	invalidate_all();
+	reach_map_changed_ = true;
+}
+
+void display::process_reachmap_changes()
+{
+	if (!reach_map_changed_) return;
+	if (reach_map_.empty() != reach_map_old_.empty()) {
+		// have to (un)brighten the inner tiles and (un)darken the outer ones
+		invalidate_all();
+	} else if (!reach_map_.empty()) {
+		// invalidate only changes
+		reach_map::iterator reach, reach_old;
+		for (reach = reach_map_.begin(); reach != reach_map_.end(); ++reach) {
+			reach_old = reach_map_old_.find(reach->first);
+			if (reach_old == reach_map_old_.end()) {
+				invalidate(reach->first);
+			} else {
+				if (reach_old->second != reach->second) {
+					invalidate(reach->first);
+				}
+				reach_map_old_.erase(reach_old);
+			}
+		}
+		for (reach_old = reach_map_old_.begin(); reach_old != reach_map_old_.end(); ++reach_old) {
+			invalidate(reach_old->first);
+		}
+	}
+	reach_map_old_ = reach_map_;
+	reach_map_changed_ = false;
 }
 
 void display::invalidate_route()
@@ -2015,13 +2047,13 @@ void display::invalidate(const gamemap::location& loc)
 			unit_map::iterator u = units_.find(loc);
 
 			if (u != units_.end()) {
-				std::set<gamemap::location> overlaps = u->second.overlaps(u->first);
+				std::set<gamemap::location> overlaps = u->second.overlaps(*this, u->first);
 				for (std::set<gamemap::location>::iterator i = overlaps.begin(); i != overlaps.end(); i++) {
 					invalidate(*i);
 				}
 			}
 			if (temp_unit_  && temp_unit_loc_ == loc ) {
-				std::set<gamemap::location> overlaps = temp_unit_->overlaps(temp_unit_loc_);
+				std::set<gamemap::location> overlaps = temp_unit_->overlaps(*this, temp_unit_loc_);
 				for (std::set<gamemap::location>::iterator i = overlaps.begin(); i != overlaps.end(); i++) {
 					invalidate(*i);
 				}
@@ -2032,13 +2064,13 @@ void display::invalidate(const gamemap::location& loc)
 			for (unsigned int i = 0; i < 6; i++) {
 				u = units_.find(adjacent[i]);
 				if (u != units_.end()) {
-					std::set<gamemap::location> overlaps = u->second.overlaps(u->first);
+					std::set<gamemap::location> overlaps = u->second.overlaps(*this, u->first);
 					if (overlaps.find(loc) != overlaps.end()) {
 						invalidate(u->first);
 					}
 				}
 				if (temp_unit_  && temp_unit_loc_ == adjacent[i] ) {
-					std::set<gamemap::location> overlaps = temp_unit_->overlaps(temp_unit_loc_);
+					std::set<gamemap::location> overlaps = temp_unit_->overlaps(*this, temp_unit_loc_);
 					if (overlaps.find(loc) != overlaps.end()) {
 						invalidate(temp_unit_loc_);
 					}
@@ -2055,11 +2087,6 @@ void display::invalidate_all()
 	update_rect(map_area());
 }
 
-void display::invalidate_unit()
-{
-	invalidateUnit_ = true;
-}
-
 void display::invalidate_animations()
 {
 	new_animation_frame();
@@ -2092,10 +2119,6 @@ void display::invalidate_animations()
 
 }
 
-void display::invalidate_theme(){
-	panelsDrawn_ = false;
-}
-
 void display::recalculate_minimap()
 {
 	if(minimap_ != NULL) {
@@ -2104,12 +2127,6 @@ void display::recalculate_minimap()
 
 	redraw_minimap();
 }
-
-void display::redraw_minimap()
-{
-	redrawMinimap_ = true;
-}
-
 void display::place_temporary_unit(unit &u, const gamemap::location& loc)
 {
 	temp_unit_ = &u;
@@ -2132,15 +2149,11 @@ void display::remove_temporary_unit()
 		invalidated_.insert(adjacent[i]);
 	}
 }
-
-void display::invalidate_game_status()
-{
-	invalidateGameStatus_ = true;
-}
-
 void display::add_overlay(const gamemap::location& loc, const std::string& img, const std::string& halo)
 {
-	const int halo_handle = halo::add(get_location_x(loc)+hex_size()/2,get_location_y(loc)+hex_size()/2,halo);
+	const int halo_handle = halo::add(get_location_x(loc) + hex_size() / 2, 
+			get_location_y(loc) + hex_size() / 2, halo, loc);
+
 	const overlay item(img,halo,halo_handle);
 	overlays_.insert(overlay_map::value_type(loc,item));
 }
@@ -2193,11 +2206,6 @@ bool display::turbo() const
 	return res || screen_.faked();
 }
 
-void display::set_turbo(bool turbo)
-{
-	turbo_ = turbo;
-}
-
 //Delay routines: use these not SDL_Delay (for --nogui).
 void display::non_turbo_delay() const
 {
@@ -2211,11 +2219,6 @@ void display::delay(unsigned int milliseconds) const
 		SDL_Delay(milliseconds);
 }
 
-void display::set_grid(bool grid)
-{
-	grid_ = grid;
-}
-
 // timestring() returns the current date as a string.
 // Uses preferences::clock_format() for formatting.
 char *timestring ( void )
@@ -2239,47 +2242,14 @@ void display::debug_highlight(const gamemap::location& loc, fixed_t amount)
 	debugHighlights_[loc] += amount;
 }
 
-void display::clear_debug_highlights()
+gui::button* display::find_button(const std::string& id)
 {
-	debugHighlights_.clear();
-}
-
-bool display::shrouded(int x, int y) const
-{
-	if(team_valid()) {
-		return teams_[currentTeam_].shrouded(x,y);
-	} else {
-		return false;
+	for (size_t i = 0; i < buttons_.size(); ++i) {
+		if(buttons_[i].id() == id) {
+			return &buttons_[i];
+		}
 	}
-}
-
-bool display::fogged(int x, int y) const
-{
-	if(team_valid()) {
-		return teams_[currentTeam_].fogged(x,y);
-	} else {
-		return false;
-	}
-}
-
-bool display::team_valid() const
-{
-	return currentTeam_ < teams_.size();
-}
-
-size_t display::viewing_team() const
-{
-	return currentTeam_;
-}
-
-size_t display::playing_team() const
-{
-	return activeTeam_;
-}
-
-theme& display::get_theme()
-{
-	return theme_;
+	return NULL;
 }
 
 const theme::menu* display::menu_pressed()
@@ -2316,6 +2286,7 @@ void display::begin_game()
 {
 	in_game_ = true;
 	create_buttons();
+	invalidate_all();
 }
 
 void display::create_buttons()
@@ -2325,6 +2296,7 @@ void display::create_buttons()
 	const std::vector<theme::menu>& buttons = theme_.menus();
 	for(std::vector<theme::menu>::const_iterator i = buttons.begin(); i != buttons.end(); ++i) {
 		gui::button b(screen_,i->title(),string_to_button_type(i->type()),i->image());
+		b.set_id(i->get_id());
 		const SDL_Rect& loc = i->location(screen_area());
 		b.set_location(loc.x,loc.y);
 		if (!i->tooltip().empty()){
@@ -2338,23 +2310,14 @@ void display::create_buttons()
 	}
 }
 
-gui::button::TYPE display::string_to_button_type(std::string type){
+gui::button::TYPE display::string_to_button_type(std::string type)
+{
 	gui::button::TYPE res = gui::button::TYPE_PRESS;
 	if (type == "checkbox") { res = gui::button::TYPE_CHECK; }
 	else if (type == "image") { res = gui::button::TYPE_IMAGE; }
 	return res;
 }
 
-void display::add_observer(const std::string& name)
-{
-	observers_.insert(name);
-}
-
-void display::remove_observer(const std::string& name)
-{
-	observers_.erase(name);
-}
-
 namespace {
 	const int chat_message_border = 5;
 	const int chat_message_x = 10;
@@ -2450,11 +2413,6 @@ void display::add_chat_message(const std::string& speaker, int side, const std::
 	}
 }
 
-void display::clear_chat_messages()
-{
-	prune_chat_messages(true);
-}
-
 void display::prune_chat_messages(bool remove_all)
 {
 	const unsigned int message_ttl = remove_all ? 0 : 1200000;
@@ -2487,15 +2445,8 @@ void display::set_diagnostic(const std::string& msg)
 	}
 }
 
-void display::rebuild_terrain(const gamemap::location &loc) {
-	builder_.rebuild_terrain(loc);
-}
-
-void display::rebuild_all() {
-	builder_.rebuild_all();
-}
-
-void display::add_highlighted_loc(const gamemap::location &hex) {
+void display::add_highlighted_loc(const gamemap::location &hex) 
+{
 	// Only invalidate and insert if this is a new addition, for
 	// efficiency.
 	if (highlighted_locations_.find(hex) == highlighted_locations_.end()) {
@@ -2504,7 +2455,8 @@ void display::add_highlighted_loc(const gamemap::location &hex) {
 	}
 }
 
-void display::clear_highlighted_locs() {
+void display::clear_highlighted_locs() 
+{
 	for (std::set<gamemap::location>::const_iterator it = highlighted_locations_.begin();
 		 it != highlighted_locations_.end(); it++) {
 		invalidate(*it);
@@ -2512,7 +2464,8 @@ void display::clear_highlighted_locs() {
 	highlighted_locations_.clear();
 }
 
-void display::remove_highlighted_loc(const gamemap::location &hex) {
+void display::remove_highlighted_loc(const gamemap::location &hex) 
+{
 	std::set<gamemap::location>::iterator it = highlighted_locations_.find(hex);
 	// Only invalidate and remove if the hex was found, for efficiency.
 	if (it != highlighted_locations_.end()) {
