@@ -1,4 +1,4 @@
-/* $Id: filesystem.cpp 8269 2005-09-17 15:06:28Z ott $ */
+/* $Id: filesystem.cpp 12753 2006-07-24 18:40:12Z sapient $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -13,6 +13,8 @@
 
 #include "global.hpp"
 
+#include "loadscreen.hpp"
+
 //include files for opendir(3), readdir(3), etc. These files may vary
 //from platform to platform, since these functions are NOT ANSI-conforming
 //functions. They may have to be altered to port to new platforms
@@ -54,6 +56,8 @@ BPath be_path;
 #include <algorithm>
 #include <fstream>
 #include <iostream>
+#include <iomanip>
+#include <sstream>
 #include <set>
 
 #include "wesconfig.h"
@@ -68,7 +72,6 @@ BPath be_path;
 #define ERR_FS LOG_STREAM(err, filesystem)
 
 #ifdef USE_ZIPIOS
-#include <sstream>
 #include <zipios++/collcoll.h>
 #include <zipios++/dircoll.h>
 #include <zipios++/zipfile.h>
@@ -140,6 +143,12 @@ namespace {
 #define DIR_INVALID(d) (d == NULL)
 #endif
 
+#ifdef __APPLE__
+#include <CoreFoundation/CoreFoundation.h>
+#include <CoreFoundation/CFString.h>
+#include <CoreFoundation/CFBase.h>
+#endif
+
 void get_files_in_dir(const std::string& directory,
                       std::vector<std::string>* files,
                       std::vector<std::string>* dirs,
@@ -202,8 +211,22 @@ void get_files_in_dir(const std::string& directory,
 		while((entry = readdir(dir)) != NULL) {
 			if(entry->d_name[0] == '.')
 				continue;
-
-			const std::string name((directory + "/") + entry->d_name);
+#ifdef __APPLE__
+		  /* HFS Mac OS X decompose filenames using combining unicode
+		  characters. Try to get the precomposed form.
+		  */
+		  char filename[MAXNAMLEN+1];
+		  CFStringRef cstr = CFStringCreateWithCString(NULL, entry->d_name,
+							       kCFStringEncodingUTF8);
+		  CFMutableStringRef mut_str = CFStringCreateMutableCopy(NULL, 0, cstr);
+		  CFStringNormalize(mut_str, kCFStringNormalizationFormC);
+		  CFStringGetCString(mut_str,filename,sizeof(filename)-1,kCFStringEncodingUTF8);
+		  CFRelease(cstr);
+		  CFRelease(mut_str);
+#else
+		  char *filename = entry->d_name;
+#endif
+			const std::string name((directory + "/") + filename);
 
 			struct stat st;
 			if (::stat(name.c_str(), &st) != -1) {
@@ -212,14 +235,14 @@ void get_files_in_dir(const std::string& directory,
 						if (mode == ENTIRE_FILE_PATH)
 							files->push_back(name);
 						else
-							files->push_back(entry->d_name);
+							files->push_back(filename);
 					}
 				} else if (S_ISDIR(st.st_mode)) {
 					if (dirs != NULL) {
 						if (mode == ENTIRE_FILE_PATH)
 							dirs->push_back(name);
 						else
-							dirs->push_back(entry->d_name);
+							dirs->push_back(filename);
 					}
 				}
 			}
@@ -283,6 +306,12 @@ std::string get_screenshot_dir()
 	return get_dir(dir_path);
 }
 
+std::string get_upload_dir()
+{
+	const std::string dir_path = get_user_data_dir() + "/upload";
+	return get_dir(dir_path);
+}
+
 std::string get_dir(const std::string& dir_path)
 {
 #ifdef _WIN32
@@ -379,6 +408,13 @@ std::string get_user_data_dir()
 		_mkdir("userdata");
 		_mkdir("userdata/editor");
 		_mkdir("userdata/editor/maps");
+		_mkdir("userdata/data");
+		_mkdir("userdata/data/ais");
+		_mkdir("userdata/data/scenarios");
+		_mkdir("userdata/data/scenarios/multiplayer");
+		_mkdir("userdata/data/maps");
+		_mkdir("userdata/data/maps/multiplayer");
+		_mkdir("userdata/saves");
 		inited_dirs = true;
 	}
 
@@ -407,9 +443,13 @@ std::string get_user_data_dir()
 	return be_path.Path();
 #else
 
+#ifndef __AMIGAOS4__
 	static const char* const current_dir = ".";
-
 	const char* home_str = getenv("HOME");
+#else
+	static const char* const current_dir = " ";
+	const char* home_str = "PROGDIR:";
+#endif
 	if(home_str == NULL)
 		home_str = current_dir;
 
@@ -419,7 +459,11 @@ std::string get_user_data_dir()
 #define PREFERENCES_DIR ".wesnoth"
 #endif
 
+#ifndef __AMIGAOS4__
 	const std::string dir_path = home + std::string("/") + PREFERENCES_DIR;
+#else
+	const std::string dir_path = home + PREFERENCES_DIR;
+#endif
 	DIR* dir = opendir(dir_path.c_str());
 	if(dir == NULL) {
 		const int res = mkdir(dir_path.c_str(),AccessMode);
@@ -427,6 +471,13 @@ std::string get_user_data_dir()
 		//also create the maps directory
 		mkdir((dir_path + "/editor").c_str(),AccessMode);
 		mkdir((dir_path + "/editor/maps").c_str(),AccessMode);
+		mkdir((dir_path + "/data").c_str(),AccessMode);
+		mkdir((dir_path + "/data/ais").c_str(),AccessMode);
+		mkdir((dir_path + "/data/scenarios").c_str(),AccessMode);
+		mkdir((dir_path + "/data/scenarios/multiplayer").c_str(),AccessMode);
+		mkdir((dir_path + "/data/maps").c_str(),AccessMode);
+		mkdir((dir_path + "/data/maps/multiplayer").c_str(),AccessMode);
+		mkdir((dir_path + "/saves").c_str(),AccessMode);
 		if(res == 0) {
 			dir = opendir(dir_path.c_str());
 		} else {
@@ -597,6 +648,39 @@ time_t file_create_time(const std::string& fname)
 	return buf.st_mtime;
 }
 
+//return the next ordered full filename within this directory
+std::string next_filename(const std::string &dirname, unsigned int max)
+{
+	std::vector<std::string> files;
+	std::stringstream fname;
+	unsigned int num = 1;
+
+	// These are sorted, so we can simply add one to last one.
+	get_files_in_dir(dirname, &files);
+
+	// Make sure we skip over any files we didn't create ourselves.
+	std::vector<std::string>::reverse_iterator i;
+	for (i = files.rbegin(); i != files.rend(); ++i) {
+		if (i->length() == 8) {
+			try {
+				num = lexical_cast<int>(*i)+1;
+				break;
+			} catch (bad_lexical_cast &) {
+			}
+		}
+	}
+
+	// Erase oldest files if we have too many
+	if (max) {
+		for (unsigned int j = 0; j + max < files.size(); j++) {
+			delete_directory(dirname + "/" + files[j]);
+		}
+	}
+
+	fname << std::setw(8) << std::setfill('0') << num;
+	return dirname + "/" + fname.str();
+}
+
 file_tree_checksum::file_tree_checksum()
     : nfiles(0), sum_size(0), modified(0)
 {}
@@ -632,7 +716,7 @@ void get_file_tree_checksum_internal(const std::string& path, file_tree_checksum
 {
 	std::vector<std::string> files, dirs;
 	get_files_in_dir(path,&files,&dirs,ENTIRE_FILE_PATH);
-
+	increment_filesystem_progress();
 	for(std::vector<std::string>::const_iterator i = files.begin(); i != files.end(); ++i) {
 		++res.nfiles;
 
