@@ -1,4 +1,4 @@
-/* $Id: game.cpp 14156 2006-11-05 11:09:59Z elias $ */
+/* $Id: game.cpp 16190 2007-03-08 22:22:48Z jhinrichs $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -658,6 +658,9 @@ bool game_controller::load_game()
 		if(campaign_define.empty() == false) {
 			defines_map_[campaign_define] = preproc_define();
 		}
+		if (campaign_define.empty() && (cfg["campaign_type"] == "multiplayer")){
+			defines_map_["MULTIPLAYER"] = preproc_define();
+		}
 
 		refresh_game_cfg();
 
@@ -1199,10 +1202,13 @@ bool game_controller::play_multiplayer()
 	preferences::set_login(login);
 
 	try {
-		defines_map_.clear();
-		defines_map_[state_.campaign_define] = preproc_define();
-		refresh_game_cfg();
+		/* do */ {
+			input_blocker eventlock;
+			defines_map_.clear();
+			defines_map_[state_.campaign_define] = preproc_define();
+			refresh_game_cfg();
 
+		}
 		if(res >= 2) {
 			std::vector<std::string> chat;
 			config game_data;
