@@ -1,5 +1,5 @@
-/* $Id: game_events.cpp 14883 2006-12-27 13:35:38Z boucman $ */
-/* $Id: game_events.cpp 14883 2006-12-27 13:35:38Z boucman $ */
+/* $Id: game_events.cpp 15943 2007-02-20 10:21:17Z noyga $ */
+/* $Id: game_events.cpp 15943 2007-02-20 10:21:17Z noyga $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -117,7 +117,11 @@ bool conditional_passed(const unit_map* units,
 
 		const std::string& equals = values["equals"];
 		if(values.get_attribute("equals") != "" && value != equals) {
-			return false;
+			//special case for uninitialized variables
+			if(!value.empty() || equals != "0")
+			{
+				return false;
+			}
 		}
 
 		const std::string& numerical_equals = values["numerical_equals"];
@@ -549,7 +553,7 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 		const size_t team_index = side_num-1;
 
 		if(team_index < teams->size()) {
-			std::cerr << "modifying team: " << team_index << "\n";
+			std::cerr << "modifying team: " << side_num << "\n";
 			if(!team_name.empty()) {
 				std::cerr << "change team to team_name '" << team_name << "'\n";
 				(*teams)[team_index].change_team(team_name,
@@ -898,7 +902,7 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 			}
 
 			size_t choice = get_random() % num_choices;
-			unsigned long tmp = 0;
+			int tmp = 0;
 			for(size_t i = 0; i < ranges.size(); i++) {
 				tmp += (ranges[i].second - ranges[i].first) + 1;
 				if (tmp > choice) {
@@ -1447,7 +1451,10 @@ bool event_handler::handle_event_command(const queued_event& event_info,
 
 		//otherwise if a choice has to be made, get it from the replay data
 		else {
-			const config* const action = get_replay_source().get_next_action();
+			const config* action = get_replay_source().get_next_action();
+			if (action != NULL && !action->get_children("start").empty()){
+				action = get_replay_source().get_next_action();
+			}
 			if(action == NULL || action->get_children("choose").empty()) {
 				replay::throw_error("choice expected but none found\n");
 			}
