@@ -1,4 +1,4 @@
-/* $Id: gettext.hpp 8646 2005-10-24 21:03:08Z ydirson $ */
+/* $Id: gettext.hpp 15612 2007-02-06 15:50:12Z torangan $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -21,16 +21,21 @@
 const char* egettext(const char*);
 const char* sgettext(const char*);
 const char* dsgettext(const char * domainname, const char *msgid);
+const char* sngettext(const char *singular, const char *plural, int n);
+const char* dsngettext(const char * domainname, const char *singular, const char *plural, int n);
 
 #ifdef GETTEXT_DOMAIN
-# define _(String) dgettext(GETTEXT_DOMAIN,String)
+# define _(String) dsgettext(GETTEXT_DOMAIN,String)
+# define _n(String1,String2,Int) dsngettext(String1,String2,Int)
 # ifdef gettext
 #  undef gettext
 # endif
 # define gettext(String) dgettext(GETTEXT_DOMAIN,String)
 # define sgettext(String) dsgettext(GETTEXT_DOMAIN,String)
+# define sngettext(String1,String2,Int) dsngettext(GETTEXT_DOMAIN,String1,String2,Int)
 #else
-# define _(String) gettext(String)
+# define _(String) sgettext(String)
+# define _n(String1,String2,Int) sngettext(String1,String2,Int)
 #endif
 
 #define gettext_noop(String) String
