@@ -1,4 +1,4 @@
-/* $Id: multiplayer.cpp 14251 2006-11-16 10:44:03Z edb $ */
+/* $Id: multiplayer.cpp 16074 2007-02-25 17:26:31Z noyga $ */
 /*
    Copyright (C)
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -12,7 +12,6 @@
 */
 
 #include "global.hpp"
-
 #include "game_config.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
@@ -61,7 +60,7 @@ void run_lobby_loop(display& disp, mp::ui& ui)
 {
 	disp.video().modeChanged();
 	bool first = true;
-
+	font::cache_mode(font::CACHE_LOBBY);
 	while (ui.get_result() == mp::ui::CONTINUE) {
 		if (disp.video().modeChanged() || first) {
 			SDL_Rect lobby_pos = { 0, 0, disp.video().getx(), disp.video().gety() };
@@ -78,6 +77,7 @@ void run_lobby_loop(display& disp, mp::ui& ui)
 		disp.flip();
 		disp.delay(20);
 	}
+	font::cache_mode(font::CACHE_GAME);
 }
 
 enum server_type {
@@ -259,14 +259,15 @@ void enter_wait_mode(display& disp, const config& game_config, game_data& data,
 
 		if (res == mp::ui::PLAY) {
 			ui.start_game();
-			if (preferences::skip_mp_replay()){
+			//FIXME commented a pointeless if since the else does exactly the same thing
+			//if (preferences::skip_mp_replay()){
 				//FIXME implement true skip replay
 				//state = ui.request_snapshot();
+			//	state = ui.get_state();
+			//}
+			//else{
 				state = ui.get_state();
-			}
-			else{
-				state = ui.get_state();
-			}
+			//}
 		}
 	}
 
