@@ -1,4 +1,4 @@
-/* $Id: multiplayer_connect.cpp 14940 2006-12-30 15:15:06Z suokko $ */
+/* $Id: multiplayer_connect.cpp 15647 2007-02-07 22:13:20Z jhinrichs $ */
 /*
    Copyright (C)
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -766,7 +766,7 @@ connect::connect(display& disp, const config& game_config, const game_data& data
 	//The number of human-controlled sides is important to know to let the server decide,
 	//how many players can join this game
 	int human_sides = 0;
-	const config::child_list cfg_sides = level_.get_children("side");
+	config::child_list cfg_sides = current_config()->get_children("side");
 	for (config::child_list::const_iterator side = cfg_sides.begin(); side != cfg_sides.end(); side++){
 		if ((**side)["controller"] == "human"){
 			human_sides++;
@@ -877,7 +877,7 @@ void connect::start_game()
 	update_and_send_diff();
 
 	// Build the gamestate object after updating the level
-	level_to_gamestate(level_, state_);
+	level_to_gamestate(level_, state_, params_.saved_game);
 
 	config cfg;
 	cfg.add_child("start_game");
@@ -1162,7 +1162,7 @@ void connect::lists_init()
     ai_algorithms_ = get_available_ais();
 
 	//Factions
-	const config::child_itors sides = level_.child_range("side");
+	const config::child_itors sides = current_config()->child_range("side");
 
 	//Teams
 	config::child_iterator sd;
@@ -1276,22 +1276,34 @@ void connect::load_game()
 			}
 		}
 
-		level_ = state_.snapshot;
+		level_["savegame"] = "yes";
+		level_["map_data"] = state_.snapshot["map_data"];
+		level_["id"] = state_.snapshot["id"];
+		level_["name"] = state_.snapshot["name"];
+		level_["turn"] = state_.snapshot["turn_at"];
+		level_["mp_use_map_settings"] = state_.snapshot["mp_use_map_settings"];
+		level_["mp_village_gold"] = state_.snapshot["mp_village_gold"];
+		level_["mp_fog"] = state_.snapshot["mp_fog"];
+		level_["mp_shroud"] = state_.snapshot["mp_shroud"];
+		level_["mp_countdown"] = state_.snapshot["mp_countdown"];
+		level_["mp_countdown_init_time"] = state_.snapshot["mp_countdown_init_time"];
+		level_["mp_countdown_turn_bonus"] = state_.snapshot["mp_countdown_turn_bonus"];
+		level_["mp_countdown_action_bonus"] = state_.snapshot["mp_countdown_action_bonus"];
+		level_["experience_modifier"] = state_.snapshot["experience_modifier"];
+		level_["observer"] = state_.snapshot["observer"];
+		level_.add_child("snapshot") = state_.snapshot;
 
 		// Adds the replay data, and the replay start, to the level, so
 		// clients can receive it.
-
-		//TO-DO: this should be done only if the player has selected for the replay to be viewed
-		if(false) {
-			level_.add_child("replay") = state_.replay_data;
-			if(!state_.starting_pos.empty())
-				level_.add_child("replay_start") = state_.starting_pos;
-		}
-
+		level_.add_child("replay") = state_.replay_data;
+		if(!state_.starting_pos.empty())
+			level_.add_child("replay_start") = state_.starting_pos;
 		level_.add_child("statistics") = statistics::write_stats();
 
 	} else {
 		level_ = params_.scenario_data;
+		level_["savegame"] = "no";
+		level_["scenario"] = params_.name;
 
 		level_["hash"] = level_.hash();
 		level_["turns"] = lexical_cast_default<std::string>(params_.num_turns, "20");
@@ -1300,23 +1312,21 @@ void connect::load_game()
 		level_["mp_countdown_turn_bonus"] = lexical_cast_default<std::string>(params_.mp_countdown_turn_bonus, "35");
 		level_["mp_countdown_reservoir_time"] = lexical_cast_default<std::string>(params_.mp_countdown_reservoir_time, "330");
 		level_["mp_countdown_action_bonus"] = lexical_cast_default<std::string>(params_.mp_countdown_action_bonus, "13");
-
-
-
-		const std::string& era = params_.era;
-
-		// Initialize the list of sides available for the current era.
-		const config* const era_cfg = game_config().find_child("era", "id", era);
-		if(era_cfg == NULL) {
-			utils::string_map i18n_symbols;
-			i18n_symbols["era"] = era;
-			throw config::error(vgettext("Cannot find era $era", i18n_symbols));
-		}
-		era_sides_ = era_cfg->get_children("multiplayer_side");
-		level_["scenario"] = params_.name;
-		level_.add_child("era", *era_cfg);
 		level_["experience_modifier"] = lexical_cast<std::string>(params_.xp_modifier);
 	}
+
+	const std::string& era = params_.era;
+
+	// Initialize the list of sides available for the current era.
+	const config* const era_cfg = game_config().find_child("era", "id", era);
+	if(era_cfg == NULL) {
+		utils::string_map i18n_symbols;
+		i18n_symbols["era"] = era;
+		throw config::error(vgettext("Cannot find era $era", i18n_symbols));
+	}
+	era_sides_ = era_cfg->get_children("multiplayer_side");
+	level_.add_child("era", *era_cfg);
+
 	gold_title_label_.hide(params_.saved_game);
 	income_title_label_.hide(params_.saved_game);
 
@@ -1336,6 +1346,21 @@ void connect::load_game()
 	}
 }
 
+config* connect::current_config(){
+	config* cfg_level = NULL;
+
+	if (level_.child("snapshot") != NULL){
+		//savegame
+		cfg_level = level_.child("snapshot");
+	}
+	else{
+		//fresh game, no snapshot available
+		cfg_level = &level_;
+	}
+
+	return cfg_level;
+}
+
 void connect::update_level()
 {
 	// Import all sides into the level
