@@ -1,4 +1,4 @@
-/* $Id: multiplayer_ui.cpp 13991 2006-10-16 12:22:13Z rusty $ */
+/* $Id: multiplayer_ui.cpp 15522 2007-02-01 22:00:47Z jhinrichs $ */
 /*
    Copyright (C) 2005
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -41,30 +41,49 @@ void check_response(network::connection res, const config& data)
 	}
 }
 
-void level_to_gamestate(config& level, game_state& state)
+void level_to_gamestate(config& level, game_state& state, bool saved_game)
 {
-	//any replay data is only temporary and should be removed from
-	//the level data in case we want to save the game later
 	config * const replay_data = level.child("replay");
 	config replay_data_store;
 	if(replay_data != NULL) {
 		replay_data_store = *replay_data;
 		LOG_NW << "setting replay\n";
+		state.replay_data = *replay_data;
 		recorder = replay(replay_data_store);
 		if(!recorder.empty()) {
 			recorder.set_skip(false);
+			recorder.set_to_end();
 		}
-
-		level.clear_children("replay");
 	}
 
 	//adds the starting pos to the level
-	if(level.child("replay_start") == NULL)
-		level.add_child("replay_start") = level;
+	if(level.child("replay_start") == NULL){
+		level.add_child("replay_start", level);
+	}
+	//this is important, if it does not happen, the starting position is missing and
+	//will be drawn from the snapshot instead (which is not what we want since we have
+	//all needed information here already)
+	state.starting_pos = *(level.child("replay_start"));
 
 	level["campaign_type"] = "multiplayer";
 	state.campaign_type = "multiplayer";
-	state.snapshot = level;
+	state.version = level["version"];
+
+	//If we start a fresh game, there won't be any snapshot information. If however this
+	//is a savegame, we got a valid snapshot here.
+	if (saved_game){
+		state.snapshot = *(level.child("snapshot"));
+
+		const config* const vars = level.child("variables");
+		if(vars != NULL) {
+			state.variables = *vars;
+		}
+		else{
+			if (state.snapshot.child("variables") != NULL){
+				state.variables = *state.snapshot.child("variables");
+			}
+		}
+	}
 
 }
 
@@ -154,6 +173,7 @@ ui::ui(display& disp, const std::string& title, const config& cfg, chat& c, conf
 	result_(CONTINUE)
 {
 	const SDL_Rect area = { 0, 0, disp.video().getx(), disp.video().gety() };
+	users_menu_.set_numeric_keypress_selection(false);
 	set_location(area);
 }
 
