@@ -1,4 +1,4 @@
-/* $Id: playcampaign.hpp 7396 2005-07-02 21:37:20Z ott $ */
+/* $Id: playcampaign.hpp 11309 2006-05-06 15:21:53Z rusty $ */
 /*
    Copyright (C) 2003-2005 by David White <davidnwhite@verizon.net>
    Copyright (C) 2005 by Philippe Plantier <ayin@anathas.org>
@@ -15,13 +15,14 @@
 #ifndef PLAYCAMPAIGN_H_INCLUDED
 #define PLAYCAMPAIGN_H_INCLUDED
 
-#include "playlevel.hpp"
+#include "menu_events.hpp"
 
 class display;
 struct game_state;
 class config;
 struct game_data;
 class CVideo;
+struct upload_log;
 
 enum io_type_t {
 	IO_NONE,
@@ -31,8 +32,13 @@ enum io_type_t {
 
 LEVEL_RESULT play_game(display& disp, game_state& state, const config& game_config,
 		const game_data& units_data, CVideo& video,
-		io_type_t io_type=IO_NONE);
+		upload_log &log,
+		io_type_t io_type=IO_NONE,
+		bool skip_replay = false);
 
 
+void play_replay(display& disp, game_state& state, const config& game_config,
+		const game_data& units_data, CVideo& video);
+
 #endif // PLAYCAMPAIGN_H_INCLUDED
 
