@@ -1,4 +1,4 @@
-/* $Id: preferences.hpp 14081 2006-10-26 10:22:11Z edb $ */
+/* $Id: preferences.hpp 14970 2007-01-01 16:44:36Z edb $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -65,6 +65,10 @@ namespace preferences {
 	bool set_sound(bool ison);
 
 	unsigned int sample_rate();
+ 	void save_sample_rate(const unsigned int rate);
+
+	size_t sound_buffer_size();
+	void save_sound_buffer_size(const size_t size);
 
 	int sound_volume();
 	void set_sound_volume(int vol);
