@@ -1,4 +1,4 @@
-/* $Id: race.hpp 7533 2005-07-20 08:22:37Z ott $ */
+/* $Id: race.hpp 11394 2006-05-10 07:37:51Z rusty $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -38,13 +38,13 @@ public:
 	bool uses_global_traits() const;
 
 	const config::child_list& additional_traits() const;
-	int num_traits() const;
+	unsigned int num_traits() const;
 
 	bool not_living() const;
 
 private:
 	t_string name_;
-	int ntraits_;
+	unsigned int ntraits_;
 	std::vector<std::string> names_[NUM_GENDERS];
 	markov_prefix_map next_[NUM_GENDERS];
 	int chain_size_;
