@@ -1,4 +1,4 @@
-/* $Id: replay.cpp 14848 2006-12-23 07:00:45Z sapient $ */
+/* $Id: replay.cpp 15947 2007-02-20 16:27:46Z martinxyz $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -690,8 +690,8 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			//replayer.set_skip(false);
 			return false;
 		}
-
-		else if(cfg->child("start") != NULL) {
+		//if there is an empty command tag, created by pre_replay() or a start tag 
+		else if( (cfg->all_children().size() == 0) || (cfg->child("start") != NULL) ){
 			//do nothing
 
 		} else if((child = cfg->child("speak")) != NULL) {
@@ -773,12 +773,7 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 				replay::throw_error(errbuf.str());
 			}
 
-			bool old_replay = false;
-			float version = atof(state_of_game.version.substr(0, 3).c_str());
-			if (version < 1.1){
-				old_replay = true;
-			}
-			unit new_unit(&gameinfo,&units,&map,&state,&teams,&(u_type->second),team_num,true, old_replay);
+			unit new_unit(&gameinfo,&units,&map,&state,&teams,&(u_type->second),team_num,true, false);
 			const std::string& res = recruit_unit(map,team_num,units,new_unit,loc);
 			if(!res.empty()) {
 				std::stringstream errbuf;
@@ -1043,6 +1038,7 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			disp.recalculate_minimap();
 			disp.invalidate_game_status();
 			disp.invalidate_all();
+			disp.draw();
 		}
 
 		child = cfg->child("verify");
