@@ -1,4 +1,4 @@
-/* $Id: game.cpp 14977 2007-01-01 18:48:02Z edb $ */
+/* $Id: game.cpp 15853 2007-02-17 09:29:08Z soliton $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -597,7 +597,7 @@ bool game::end_turn()
 
 	const size_t nsides = level_.get_children("side").size();
 
-	if((end_turn_%nsides) != 0) {
+	if(nsides == 0 || (end_turn_%nsides) != 0) {
 		return false;
 	}
 
