@@ -1,4 +1,4 @@
-/* $Id: player.hpp 7532 2005-07-20 07:51:47Z ott $ */
+/* $Id: player.hpp 11323 2006-05-07 03:30:52Z rusty $ */
 /*
    Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -14,6 +14,8 @@
 #ifndef PLAYER_HPP_INCLUDED
 #define PLAYER_HPP_INCLUDED
 
+#include <ctime>
+
 #include "../config.hpp"
 
 #include <string>
@@ -21,17 +23,25 @@
 class player
 {
 public:
-	player(const std::string& n, config& cfg);
+	player(const std::string& n, config& cfg,size_t max_messages=4,size_t time_period=10);
 
-	void mark_available(bool val);
+	void mark_available(bool val,std::string location);
 
 	const std::string& name() const;
 
 	config* config_address();
 
+	bool silenced() const;
+	bool is_message_flooding();
+
 private:
 	std::string name_;
 	config& cfg_;
+
+	time_t flood_start_;
+	unsigned int messages_since_flood_start_;
+	size_t MaxMessages;
+	time_t TimePeriod;
 };
 
 #endif
