@@ -1,4 +1,4 @@
-/* $Id: show_dialog.cpp 8514 2005-10-09 20:59:44Z dave $ */
+/* $Id: show_dialog.cpp 11809 2006-06-03 08:14:03Z sapient $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -19,14 +19,13 @@
 #include "cursor.hpp"
 #include "display.hpp"
 #include "events.hpp"
-#include "font.hpp"
 #include "gettext.hpp"
 #include "help.hpp"
 #include "hotkeys.hpp"
 #include "image.hpp"
 #include "key.hpp"
 #include "log.hpp"
-#include "playlevel.hpp"
+#include "marked-up_text.hpp"
 #include "show_dialog.hpp"
 #include "thread.hpp"
 #include "language.hpp"
@@ -40,7 +39,7 @@
 #include "widgets/textbox.hpp"
 #include "wassert.hpp"
 
-#include "SDL_ttf.h"
+#include "sdl_ttf/SDL_ttf.h"
 
 #include <iostream>
 #include <numeric>
@@ -324,17 +323,17 @@ int show_dialog(display& disp, surface image,
 				int text_widget_max_chars,
 				dialog_action* action, std::vector<check_item>* options, int xloc, int yloc,
 				const std::string* dialog_style, std::vector<dialog_button>* action_buttons,
-				const std::string& help_topic, const menu::sorter* sorter)
+				const std::string& help_topic, const menu::sorter* sorter, menu::style* menu_style)
 {
 	if(disp.update_locked())
 		return -1;
 
 	LOG_DP << "showing dialog '" << caption << "' '" << msg << "'\n";
 
-	//create the event context, but only activate it if we don't have preview panes.
-	//the presence of preview panes indicates that the caller will create the context,
-	//so that their preview panes may fall inside it.
-	const events::event_context dialog_events_context(preview_panes == NULL);
+	//create the event context, remember to instruct any passed-in widgets to join it
+	//FIXME outer_context is needed to prevent delayed event_context instantiation(?)
+	const events::event_context outer_context;
+	const events::event_context dialog_events_context;
 	const dialog_manager manager;
 
 	const events::resize_lock prevent_resizing;
@@ -379,7 +378,7 @@ int show_dialog(display& disp, surface image,
 #else
 	const int max_menu_width = -1;
 #endif
-	menu menu_(screen,menu_items,type == MESSAGE,-1,max_menu_width,sorter);
+	menu menu_(screen,menu_items,type == MESSAGE,-1,max_menu_width,sorter,menu_style);
 
 	menu_.set_numeric_keypress_selection(use_textbox == false);
 
@@ -389,9 +388,12 @@ int show_dialog(display& disp, surface image,
 	const size_t top_padding = font::relative_size(10);
 	const size_t bottom_padding = font::relative_size(10);
 
-	const std::string message = font::word_wrap_text(msg, message_font_size,
-			screen.getx() / 2,
-			screen.gety() / 2);
+	std::string message = "";
+	try {
+		message = font::word_wrap_text(msg, message_font_size, screen.getx() / 2, screen.gety() / 2);
+	} catch(utils::invalid_utf8_exception&) {
+		ERR_DP << "Problem handling utf8 in message '" << msg << "'\n";
+	}
 
 	SDL_Rect text_size = { 0, 0, 0, 0 };
 	if(!message.empty()) {
@@ -466,6 +468,7 @@ int show_dialog(display& disp, surface image,
 	if(options != NULL) {
 		for(std::vector<check_item>::const_iterator i = options->begin(); i != options->end(); ++i) {
 			button check_button(screen,i->label,button::TYPE_CHECK);
+			check_button.set_align(i->align);
 			check_button_height += check_button.height() + button_height_padding;
 			check_button_width = maximum<int>(check_button.width(),check_button_width);
 
@@ -487,6 +490,7 @@ int show_dialog(display& disp, surface image,
 	size_t preview_pane_height = 0, left_preview_pane_width = 0, right_preview_pane_width = 0;
 	if(preview_panes != NULL) {
 		for(std::vector<preview_pane*>::const_iterator i = preview_panes->begin(); i != preview_panes->end(); ++i) {
+			(**i).join(); //join the event context now, since it was created outside this scope
 			const SDL_Rect& rect = (**i).location();
 
 			if((**i).show_above() == false) {
@@ -528,6 +532,11 @@ int show_dialog(display& disp, surface image,
 
 	if(text_widget_width+left_padding+right_padding > total_width)
 		total_width = text_widget_width+left_padding+right_padding;
+	//Prevent the menu from being too skinny
+	if(menu_.width() > 0 && preview_panes == NULL &&
+		total_width > menu_.width() + image_width + padding_width) {
+		menu_.set_width(total_width - image_width - padding_width);
+	}
 
 	const size_t text_and_image_height = image_height > total_text_height ? image_height : total_text_height;
 
@@ -633,9 +642,12 @@ int show_dialog(display& disp, surface image,
 		menu_.set_location(menu_xpos,menu_ypos);
 	}
 
+	text_size.x = xloc + left_padding;
+	text_size.y = yloc + top_padding + caption_size.h;
 	if(image != NULL) {
 		const int x = xloc + left_padding;
 		const int y = yloc + top_padding;
+		text_size.x += image_width + image_h_padding;
 
 		screen.blit_surface(x,y,image);
 
@@ -645,17 +657,21 @@ int show_dialog(display& disp, surface image,
 		                yloc+top_padding);
 	}
 
-	font::draw_text(&screen, clipRect, message_font_size,
+	font::draw_text(&screen, text_size, message_font_size,
 	                font::NORMAL_COLOUR, message,
 	                xloc+image_width+left_padding+image_h_padding,
 	                yloc+top_padding+caption_size.h);
 
-	//set the position of any tick boxes. they go right below the menu, slammed against
-	//the right side of the dialog
+	//set the position of any tick boxes. by default, they go right below the menu,
+	//slammed against the right side of the dialog
 	if(check_buttons.empty() == false) {
 		int options_y = text_widget_y + text_widget_height + menu_.height() + button_height_padding + menu_hpadding;
 		for(size_t i = 0; i != check_buttons.size(); ++i) {
-			check_buttons[i].set_location(xloc + total_width - check_buttons[i].width() - ButtonHPadding,options_y);
+			if(check_buttons[i].align() == LEFT_ALIGN) {
+				check_buttons[i].set_location(xloc + ButtonHPadding,options_y);
+			} else {
+				check_buttons[i].set_location(xloc + total_width - check_buttons[i].width() - ButtonHPadding,options_y);
+			}
 
 			options_y += check_buttons[i].height() + button_height_padding;
 
@@ -817,7 +833,7 @@ int show_dialog(display& disp, surface image,
 		}
 
 		disp.flip();
-		SDL_Delay(10);
+		disp.delay(10);
 
 		if(action != NULL) {
 			const int act = action->do_action();
@@ -830,14 +846,43 @@ int show_dialog(display& disp, surface image,
 	return -1;
 }
 
+/*	show_dialog2: same as show_dialog, but uses the image-select style
+	provided as a convenience to use default parameters */
+//inline
+int show_dialog2(display &screen, surface image,
+				const std::string& caption, const std::string& message,
+				DIALOG_TYPE type,
+				const std::vector<std::string>* menu_items,
+				const std::vector<preview_pane*>* preview_panes,
+				const std::string& text_widget_label,
+				std::string* text_widget_text,
+				const int text_widget_max_chars,
+				dialog_action* action,
+				std::vector<check_item>* options, int xloc, int yloc,
+				const std::string* dialog_style,
+				std::vector<dialog_button>* buttons,
+				const std::string& help_topic,
+				const menu::sorter* sorter
+			 )
+{
+	return show_dialog(screen, image,
+				caption, message,
+				type, menu_items,
+				preview_panes,
+				text_widget_label,
+				text_widget_text,
+				text_widget_max_chars,
+				action, options, xloc, yloc,
+				dialog_style, buttons,
+				help_topic, sorter, &menu::bluebg_style);
+}
+
 }
 
 namespace gui {
 
-network::connection network_data_dialog(display& disp, const std::string& msg, config& cfg, network::connection connection_num)
+	network::connection network_data_dialog(display& disp, const std::string& msg, config& cfg, network::connection connection_num, network::statistics (*get_stats)(network::connection handle))
 {
-	const std::string title = _("Receiving data...");
-
 	const size_t width = 300;
 	const size_t height = 80;
 	const size_t border = 20;
@@ -850,7 +895,7 @@ network::connection network_data_dialog(display& disp, const std::string& msg, c
 	std::vector<gui::button*> buttons_ptr(1,&cancel_button);
 
 	surface_restorer restorer;
-	gui::draw_dialog(left,top,width,height,disp.video(),title,NULL,&buttons_ptr,&restorer);
+	gui::draw_dialog(left,top,width,height,disp.video(),msg,NULL,&buttons_ptr,&restorer);
 
 	const SDL_Rect progress_rect = {left+border,top+border,width-border*2,height-border*2};
 	gui::progress_bar progress(disp.video());
@@ -859,12 +904,12 @@ network::connection network_data_dialog(display& disp, const std::string& msg, c
 	events::raise_draw_event();
 	disp.flip();
 
-	network::statistics old_stats = network::get_receive_stats(connection_num);
+	network::statistics old_stats = get_stats(connection_num);
 
 	cfg.clear();
 	for(;;) {
 		const network::connection res = network::receive_data(cfg,connection_num,100);
-		const network::statistics stats = network::get_receive_stats(connection_num);
+		const network::statistics stats = get_stats(connection_num);
 		if(stats.current_max != 0 && stats != old_stats) {
 			old_stats = stats;
 			progress.set_progress_percent((stats.current*100)/stats.current_max);
@@ -887,6 +932,18 @@ network::connection network_data_dialog(display& disp, const std::string& msg, c
 	}
 }
 
+network::connection network_send_dialog(display& disp, const std::string& msg, config& cfg, network::connection connection_num)
+{
+	return network_data_dialog(disp, msg, cfg, connection_num,
+							   network::get_send_stats);
+}
+
+network::connection network_receive_dialog(display& disp, const std::string& msg, config& cfg, network::connection connection_num)
+{
+	return network_data_dialog(disp, msg, cfg, connection_num,
+							   network::get_receive_stats);
+}
+
 namespace {
 
 class connect_waiter : public threading::waiter
