@@ -1,4 +1,4 @@
-/* $Id: team.hpp 13883 2006-10-10 14:15:31Z edb $ */
+/* $Id: team.hpp 14883 2006-12-27 13:35:38Z boucman $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@verizon.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -73,6 +73,7 @@ public:
 		std::vector<std::string> recruitment_pattern;
 		std::vector<int> enemies;
 		std::string team_name;
+		std::string user_team_name;
 		std::string save_id;
 		std::string current_player;
 		std::string countdown_time;
@@ -185,7 +186,9 @@ public:
 	void make_ai();
 
 	const std::string& team_name() const;
-	void change_team(const std::string& name);
+	const std::string& user_team_name() const;
+	void change_team(const std::string& name,
+					 const std::string& user_name);
 
 	const std::string& flag() const;
 
