@@ -1,11 +1,25 @@
-#include "global.hpp"
+/* $Id: thread.cpp 8259 2005-09-16 21:41:53Z j_daniel $ */
+/*
+   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY.
+
+   See the COPYING file for more details.
+*/
+
+#include "global.hpp"
+#include "log.hpp"
 #include "thread.hpp"
 
 #include <new>
 #include <iostream>
 #include <vector>
 
+#define ERR_G LOG_STREAM(err, general)
 namespace {
 
 int run_async_operation(void* data)
@@ -106,28 +120,39 @@ bool condition::wait(const mutex& m)
 
 condition::WAIT_TIMEOUT_RESULT condition::wait_timeout(const mutex& m, unsigned int timeout)
 {
-	const int res = SDL_CondWaitTimeout(cond_,m.m_,timeout) == 0;
+	const int res = SDL_CondWaitTimeout(cond_,m.m_,timeout);
 	switch(res) {
-	//the SDL documentation appears backward on when these results are returned
-	case 0: return WAIT_TIMEOUT;
-	case SDL_MUTEX_TIMEDOUT: return WAIT_OK;
-	default: return WAIT_ERROR;
+		case 0: return WAIT_OK;
+		case SDL_MUTEX_TIMEDOUT: return WAIT_TIMEOUT;
+		default:
+			ERR_G << "SDL_CondWaitTimeout: " << SDL_GetError() << "\n";
+			return WAIT_ERROR;
 	}
 }
 
-void condition::notify_one()
+bool condition::notify_one()
 {
-	SDL_CondSignal(cond_);
+	if(SDL_CondSignal(cond_) < 0) {
+		ERR_G << "SDL_CondSignal: " << SDL_GetError() << "\n";
+		return false;
+	}
+
+	return true;
 }
 
-void condition::notify_all()
+bool condition::notify_all()
 {
-	SDL_CondBroadcast(cond_);
+	if(SDL_CondBroadcast(cond_) < 0) {
+		ERR_G << "SDL_CondBroadcast: " << SDL_GetError() << "\n";
+		return false;
+	}
+	return true;
 }
 
-void async_operation::notify_finished()
+bool async_operation::notify_finished()
 {
-	finished_.notify_one();
+	finishedVar_ = true;
+	return finished_.notify_one();
 }
 
 async_operation::RESULT async_operation::execute(waiter& wait)
@@ -147,12 +172,15 @@ async_operation::RESULT async_operation::execute(waiter& wait)
 		bool completed = false;
 		while(wait.process() == waiter::WAIT) {
 			const condition::WAIT_TIMEOUT_RESULT res = finished_.wait_timeout(get_mutex(),20);
-			if(res == condition::WAIT_OK) {
+			if(res == condition::WAIT_OK || finishedVar_) {
 				completed = true;
 				break;
-			} else if(res == condition::WAIT_ERROR) {
+			}
+#ifndef __BEOS__
+			else if(res == condition::WAIT_ERROR) {
 				break;
 			}
+#endif
 		}
 
 		if(!completed) {
